package main

import (
	"bufio"
	"errors"
	"fmt"
	"io"
	"log"
	"os"
)

const fname = "/home/rob/home-office/example.mp3"

type (
	AudioVersionId int
	LayerIndex     int

	header uint32
)

const (
	AudioVersionMPEG25 AudioVersionId = iota
	AudioVersionReserved
	AudioVersionMPEG2
	AudioVersionMPEG1
)

const (
	LayerIndexReserved LayerIndex = iota
	LayerIndexIII
	LayerIndexII
	LayerIndexI
)

var sampleRates = [5][16]uint32{
	{0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0},
	{0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0},
	{0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0},
	{0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0},
	{0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0},
}

func (h header) AudioVersionId() AudioVersionId {
	return AudioVersionId(((h >> 8) & 0x18) >> 3)
}

func (h header) LayerIndex() LayerIndex {
	return LayerIndex(((h >> 8) & 0x6) >> 1)
}

func (h header) IsPrivate() bool {
	return (h>>8)&1 == 1
}

func (h header) SampleRate() uint32 {
	var j int
	switch h.AudioVersionId() {
	case AudioVersionMPEG1:
		switch h.LayerIndex() {
		case LayerIndexIII:
			j = 2
		case LayerIndexII:
			j = 1
		case LayerIndexI:
			j = 0
		default:
			return 0
		}
	case AudioVersionReserved:
		return 0
	case AudioVersionMPEG2, AudioVersionMPEG25:
		switch h.LayerIndex() {
		case LayerIndexIII, LayerIndexII:
			j = 4
		case LayerIndexI:
			j = 3
		default:
			return 0

		}
	}
	k := ((h >> 20) & 0xf)
	return sampleRates[j][k]
}

func main() {
	f, err := os.Open(fname)
	if err != nil {
		log.Fatal(err)
	}

	bufreader := bufio.NewReader(f)
	buf := make([]byte, 4)

	var i int
	for {
		_, err := io.ReadFull(bufreader, buf)
		if err != nil {
			if err == io.EOF {
				break
			}
			log.Fatal(err)
		}

		header, err := newHeaderFromBytes(buf)
		if err == nil {
			fmt.Printf("Got header: %08X, audio version: %d, layer index: %d\n", header, header.AudioVersionId(), header.LayerIndex())
		}
		i++

		bufreader.Discard(3)
	}
}

func newHeaderFromBytes(buf []byte) (header, error) {
	if len(buf) < 4 {
		return 0, errors.New("insufficient bytes")
	}
	if buf[0] != 0xFF || buf[1]&0xE0 != 0xE0 {
		return 0, errors.New("no header")
	}

	// always little-endian for now
	n := uint32(buf[0]) | uint32(buf[1])<<8 | uint32(buf[2])<<16 | uint32(buf[3])<<24
	return newHeader(n)
}

// TODO use unsafe here? https://codereview.stackexchange.com/a/60161
func newHeader(n uint32) (header, error) {
	return header(n), nil
}
