package main

import (
	"encoding/binary"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestAudioVersionId(t *testing.T) {
	bytes := []byte{0xff, 0xe8, 0, 0}
	h, err := newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.Equal(t, AudioVersionReserved, h.AudioVersionId())

	bytes = []byte{0xff, 0xf0, 0, 0}
	h, err = newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.Equal(t, AudioVersionMPEG2, h.AudioVersionId())

	bytes = []byte{0xff, 0xff, 0, 0}
	h, err = newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.Equal(t, AudioVersionMPEG1, h.AudioVersionId())

	bytes = []byte{0xff, 0xe0, 0, 0}
	h, err = newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.Equal(t, AudioVersionMPEG25, h.AudioVersionId())
}

func TestLayerIndex(t *testing.T) {
	bytes := []byte{0xff, 0xe2, 0, 0}
	h, err := newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.Equal(t, LayerIndexIII, h.LayerIndex())

	bytes = []byte{0xff, 0xe4, 0, 0}
	h, err = newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.Equal(t, LayerIndexII, h.LayerIndex())

	bytes = []byte{0xff, 0xe6, 0, 0}
	h, err = newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.Equal(t, LayerIndexI, h.LayerIndex())
}

func TestIsPrivate(t *testing.T) {
	bytes := []byte{0xff, 0xe1, 0, 0}
	h, err := newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.True(t, h.IsPrivate())

	bytes = []byte{0xff, 0xe0, 0, 0}
	h, err = newHeader(binary.LittleEndian.Uint32(bytes))
	require.NoError(t, err)
	assert.False(t, h.IsPrivate())
}

func TestSampleRate(t *testing.T) {
	testCases := []struct {
		name          string
		bytes         []byte
		expSampleRate uint32
	}{
		{
			"empty header",
			[]byte{0, 0, 0, 0},
			0,
		},
		{
			"MPEG1 LayerI 32kbps",
			[]byte{0xff, 0xfe, 0x10, 0},
			32,
		},
		{
			"MPEG1 LayerI 448kbps",
			[]byte{0xff, 0xfe, 0xe0, 0},
			448,
		},
		{
			"MPEG1 LayerII 32kbps",
			[]byte{0xff, 0xfc, 0x10, 0},
			32,
		},
		{
			"MPEG1 LayerII 448kbps",
			[]byte{0xff, 0xfc, 0xe0, 0},
			384,
		},
		{
			"MPEG1 LayerIII 32kbps",
			[]byte{0xff, 0xfa, 0x10, 0},
			32,
		},
		{
			"MPEG1 LayerIII 128kbps",
			[]byte{0xff, 0xfa, 0x90, 0},
			128,
		},
		{
			"MPEG1 LayerIII 160kbps",
			[]byte{0xff, 0xfa, 0xa0, 0},
			160,
		},
		{
			"MPEG1 LayerIII 320kbps",
			[]byte{0xff, 0xfa, 0xe0, 0x0},
			320,
		},
		{
			"MPEG1 LayerIII reserved",
			[]byte{0xff, 0xfa, 0xf0, 0x0},
			0,
		},
	}

	for _, tc := range testCases {
		t.Run(tc.name, func(t *testing.T) {
			h, err := newHeader(binary.LittleEndian.Uint32(tc.bytes))
			require.NoError(t, err)
			assert.Equal(t, tc.expSampleRate, h.SampleRate())
		})
	}
}
